"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CacheHelper = void 0;
const typescript_helper_functions_1 = require("typescript-helper-functions");
const NodeCache = require("node-cache");
class CacheHelper {
    /**
     * Create new instance of CacheHelper
     * @param cacheNames {string[]} List of cache names
     * @param cacheExpirationSeconds {number} Time to expiration for the cache in seconds
     */
    constructor(cacheNames, cacheExpirationSeconds = 300) {
        this.ObjectOperations = new typescript_helper_functions_1.ObjectOperations();
        if (this.ObjectOperations.IsNullOrEmpty(cacheNames))
            throw new Error(`Value cannot be null or undefined: [cacheNames]`);
        if (cacheNames.length < 1)
            throw new Error(`Value cannot be null or undefined: [cacheNames]`);
        this.Caches = [];
        this.Initialize(cacheNames, cacheExpirationSeconds);
    }
    /**
     * Store item in cache
     * @param cacheName {string} Cache name
     * @param key {string} Cache item key
     * @param value {string} Cache item value
     */
    Add(cacheName, key, value) {
        const action = `${CacheHelper.name}.${this.Add.name}`;
        if (this.ObjectOperations.IsNullOrWhitespace(cacheName))
            throw new Error(`[${action}]-Value cannot be null or undefined: [cacheName]`);
        if (this.ObjectOperations.IsNullOrWhitespace(key))
            throw new Error(`[${action}]-Value cannot be null or undefined: [key]`);
        if (this.ObjectOperations.IsNullOrEmpty(value))
            throw new Error(`[${action}]-Value cannot be null or undefined: [value]`);
        const cache = this.FindCache(cacheName);
        return cache.set(key, value);
    }
    /**
     * Delete item in cache
     * @param cacheName {string} Cache name
     * @param key {string} Cache item key
     */
    Delete(cacheName, key) {
        const action = `${CacheHelper.name}.${this.Delete.name}`;
        if (this.ObjectOperations.IsNullOrWhitespace(cacheName))
            throw new Error(`[${action}]-Value cannot be null or undefined: [cacheName]`);
        if (this.ObjectOperations.IsNullOrWhitespace(key))
            throw new Error(`[${action}]-Value cannot be null or undefined: [key]`);
        const cache = this.FindCache(cacheName);
        const deleted = cache.del(key);
        return deleted > 0;
    }
    /**
     * Find cache
     * @param cacheName {string} Cache name
     */
    FindCache(cacheName) {
        const action = `${CacheHelper.name}.${this.FindCache.name}`;
        if (this.ObjectOperations.IsNullOrWhitespace(cacheName))
            throw new Error(`[${action}]-Value cannot be null or undefined: [cacheName]`);
        const cache = this.Caches.find(c => c.Name === cacheName);
        if (this.ObjectOperations.IsNullOrEmpty(cache)) {
            throw new Error('Cache not found');
        }
        else {
            return cache.NodeCache;
        }
    }
    /**
     * Flush cache
     * @param cacheName {string} Cache name
     */
    Flush(cacheName) {
        const action = `${CacheHelper.name}.${this.Flush.name}`;
        if (this.ObjectOperations.IsNullOrWhitespace(cacheName))
            throw new Error(`[${action}]-Value cannot be null or undefined: [cacheName]`);
        const cache = this.FindCache(cacheName);
        cache.flushAll();
    }
    /**
     * Get item from cache
     * @param cacheName {string} Cache name
     * @param key {string} Cache item key
     */
    Get(cacheName, key) {
        const action = `${CacheHelper.name}.${this.Get.name}`;
        if (this.ObjectOperations.IsNullOrWhitespace(cacheName))
            throw new Error(`[${action}]-Value cannot be null or undefined: [cacheName]`);
        if (this.ObjectOperations.IsNullOrWhitespace(key))
            throw new Error(`[${action}]-Value cannot be null or undefined: [key]`);
        const cache = this.FindCache(cacheName);
        return cache.get(key);
    }
    /**
     * Initialize the caches
     * @param cacheName {string} Cache name
     * @param cacheExpirationSeconds {number} Time to expiration for the cache in seconds
     */
    Initialize(cacheNames, cacheExpirationSeconds) {
        const action = `${CacheHelper.name}.${this.Initialize.name}`;
        if (this.ObjectOperations.IsNullOrEmpty(cacheNames))
            throw new Error(`[${action}]-Value cannot be null or undefined: [cacheNames]`);
        const options = {
            stdTTL: cacheExpirationSeconds,
        };
        for (const cacheName of cacheNames) {
            this.Caches.push({
                Name: cacheName,
                NodeCache: new NodeCache(options),
            });
        }
    }
    /**
     * Replace item in cache
     * @param cacheName {string} Cache name
     * @param key {string} Cache item key
     * @param value {object} Cache item value
     */
    Replace(cacheName, key, value) {
        const action = `${CacheHelper.name}.${this.Replace.name}`;
        if (this.ObjectOperations.IsNullOrWhitespace(cacheName))
            throw new Error(`[${action}]-Value cannot be null or undefined: [cacheName]`);
        if (this.ObjectOperations.IsNullOrWhitespace(key))
            throw new Error(`[${action}]-Value cannot be null or undefined: [key]`);
        if (this.ObjectOperations.IsNullOrEmpty(value))
            throw new Error(`[${action}]-Value cannot be null or undefined: [value]`);
        const cacheItem = this.Get(cacheName, key);
        if (!this.ObjectOperations.IsNullOrEmpty(cacheItem)) {
            this.Delete(cacheName, key);
        }
        this.Add(cacheName, key, value);
    }
}
exports.CacheHelper = CacheHelper;
//# sourceMappingURL=helper.js.map